function [MeanResults, StdResults] = HedwigExpSim(step);

%% simulating Hedwig experiment over length trun

close all

%alpha = 0.008;  % Bonferroni correction
%z = 2.668; % used to calculate CIs and Bonferroni correction has been applied
transient = round(31/4); % initial transient of transient time steps of each stochastic run: 31 days = 31/4 time steps
tlimit = 37; % Number of time steps within each stochastic run
trun = transient+tlimit; 
index=linspace(1,149,tlimit+1); index=index(1:end-1); % index for census days

% retrieve environmental sequences used in experiment
[Bperday, Wperday, Rperday, EnvHarvest] = EnvExp(transient, tlimit, trun);

% DEB IPM pars 
Lmin = 150/1000; % minimum body length in mm (used to define limits of matrix approximation)
Lb = 166/1000; % length at birth in mm 
Lm = 1008/1000; % maximum body length in mm 
E_Ystdev=0.3; % sigma(Y)
E_Ymax = linspace(0.36,0.6,step); %% feeding level representing 8 yeast rods per day
E_Ymin = linspace(0.1,0.36,step); % feeding level representing 2 yeast rods per day
kappa = 0.082; % fraction energy allocation to respiration (as opposed to reproduction)
mu = 0.03; % mortality rate
beta = 151; % to calculate von Bertalanffy growth rate r_b
alpha = -137.8; % to calculate von Bertalanffy growth rate r_b
Rm = 32; % maximum reproduction rate of female offspring at L_m

% IPM approximation pars
MatrixSize = 200; % number of size bins of matrix approximation of DEBIPM
nbins = MatrixSize;

for i=1:length(E_Ymin), i/length(E_Ymin) 
    for j=1:length(E_Ymax)
    
        %calculate population dynamics white environment no harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseq(Wperday,MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4); % *4 because one time step equals four days 
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanW(i,j)=mean(log(numberTotal)); popstdW(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        %[acf,lags,bounds] = autocorr(log(numberTotal),1); autocorrW(i,j) = acf(2); 
        eggmeannumberW(i,j)=mean(log(numberE(index))); juvsmeannumberW(i,j)=mean(log(numberJ(index))); adultmeannumberW(i,j)=mean(log(numberA(index)));
        eggstdW(i,j)=std(log(numberE(index)));juvstdW(i,j)=std(log(numberJ(index)));adultstdW(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthW(i,j)=mean(log(growtheggs)); juvsmeangrowthW(i,j)=mean(log(growthjuvs)); adultmeangrowthW(i,j)=mean(log(growthadults));
        eggstdgrowthW(i,j)=std(log(growtheggs));juvstdW(i,j)=std(log(growthjuvs));adultstdW(i,j)=std(log(growthadults));
        meaneggpropW(i,j) = mean(numberE(index)./numberTotal); meanjuvpropW(i,j) = mean(numberJ(index)./numberTotal); meanadultpropW(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthW(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));
        %[acf,lags,bounds] = autocorr(log(numberE(index)),1); autocorrWE(i,j) = acf(2); 
        %[acf,lags,bounds] = autocorr(log(numberJ(index)),1); autocorrWJ(i,j) = acf(2); 
        %[acf,lags,bounds] = autocorr(log(numberA(index)),1); autocorrWA(i,j) = acf(2); 
        
        clear numberE numberJ numberA numberTotal
        %calculate population dynamics blue environment no harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseq(Bperday, MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4);
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanB(i,j)=mean(log(numberTotal)); popstdB(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        eggmeannumberB(i,j)=mean(log(numberE(index))); juvsmeannumberB(i,j)=mean(log(numberJ(index))); adultmeannumberB(i,j)=mean(log(numberA(index)));
        eggstdB(i,j)=std(log(numberE(index)));juvstdB(i,j)=std(log(numberJ(index)));adultstdB(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthB(i,j)=mean(log(growtheggs)); juvsmeangrowthB(i,j)=mean(log(growthjuvs)); adultmeangrowthB(i,j)=mean(log(growthadults));
        eggstdgrowthB(i,j)=std(log(growtheggs));juvstdB(i,j)=std(log(growthjuvs));adultstdB(i,j)=std(log(growthadults));
        meaneggpropB(i,j) = mean(numberE(index)./numberTotal); meanjuvpropB(i,j) = mean(numberJ(index)./numberTotal); meanadultpropB(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthB(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));
        
        clear numberE numberJ numberA numberTotal
        %calculate population dynamics red environment no harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseq(Rperday, MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4);
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanR(i,j)=mean(log(numberTotal)); popstdR(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        eggmeannumberR(i,j)=mean(log(numberE(index))); juvsmeannumberR(i,j)=mean(log(numberJ(index))); adultmeannumberR(i,j)=mean(log(numberA(index)));
        eggstdR(i,j)=std(log(numberE(index)));juvstdR(i,j)=std(log(numberJ(index)));adultstdR(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthR(i,j)=mean(log(growtheggs)); juvsmeangrowthR(i,j)=mean(log(growthjuvs)); adultmeangrowthR(i,j)=mean(log(growthadults));
        eggstdgrowthR(i,j)=std(log(growtheggs));juvstdR(i,j)=std(log(growthjuvs));adultstdR(i,j)=std(log(growthadults));
        meaneggpropR(i,j) = mean(numberE(index)./numberTotal); meanjuvpropR(i,j) = mean(numberJ(index)./numberTotal); meanadultpropR(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthR(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));

        clear numberE numberJ numberA numberTotal
        %calculate population dynamics white environment WITH harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseqHarvest(Wperday,EnvHarvest, MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4); % *4 because one time step equals four days 
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanWH(i,j)=mean(log(numberTotal)); popstdWH(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        eggmeannumberWH(i,j)=mean(log(numberE(index))); juvsmeannumberWH(i,j)=mean(log(numberJ(index))); adultmeannumberWH(i,j)=mean(log(numberA(index)));
        eggstdWH(i,j)=std(log(numberE(index)));juvstdWH(i,j)=std(log(numberJ(index)));adultstdWH(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthWH(i,j)=mean(log(growtheggs)); juvsmeangrowthWH(i,j)=mean(log(growthjuvs)); adultmeangrowthWH(i,j)=mean(log(growthadults));
        eggstdgrowthWH(i,j)=std(log(growtheggs));juvstdWH(i,j)=std(log(growthjuvs));adultstdWH(i,j)=std(log(growthadults));
        meaneggpropWH(i,j) = mean(numberE(index)./numberTotal); meanjuvpropWH(i,j) = mean(numberJ(index)./numberTotal); meanadultpropWH(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthWH(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));

        clear numberE numberJ numberA numberTotal
        %calculate population dynamics blue environment WITH harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseqHarvest(Bperday,EnvHarvest, MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4); % *4 because one time step equals four days 
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanBH(i,j)=mean(log(numberTotal)); popstdBH(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        eggmeannumberBH(i,j)=mean(log(numberE(index))); juvsmeannumberBH(i,j)=mean(log(numberJ(index))); adultmeannumberBH(i,j)=mean(log(numberA(index)));
        eggstdBH(i,j)=std(log(numberE(index)));juvstdBH(i,j)=std(log(numberJ(index)));adultstdBH(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthBH(i,j)=mean(log(growtheggs)); juvsmeangrowthBH(i,j)=mean(log(growthjuvs)); adultmeangrowthBH(i,j)=mean(log(growthadults));
        eggstdgrowthBH(i,j)=std(log(growtheggs));juvstdBH(i,j)=std(log(growthjuvs));adultstdBH(i,j)=std(log(growthadults));
        meaneggpropBH(i,j) = mean(numberE(index)./numberTotal); meanjuvpropBH(i,j) = mean(numberJ(index)./numberTotal); meanadultpropBH(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthBH(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));

        clear numberE numberJ numberA numberTotal
        %calculate population dynamics red environment WITH harvesting
        [numberE, numberJ, numberA, numberTotal] = stochseqHarvest(Rperday,EnvHarvest, MatrixSize,E_Ymax(j),E_Ymin(i),Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient*4,tlimit*4); % *4 because one time step equals four days 
        numberTotal=numberTotal(index); % number calculated over 4 days = census step
        popmeanRH(i,j)=mean(log(numberTotal)); popstdRH(i,j)=std(log(numberTotal)); % lambda_s and std in lambda_s
        eggmeannumberRH(i,j)=mean(log(numberE(index))); juvsmeannumberRH(i,j)=mean(log(numberJ(index))); adultmeannumberRH(i,j)=mean(log(numberA(index)));
        eggstdRH(i,j)=std(log(numberE(index)));juvstdRH(i,j)=std(log(numberJ(index)));adultstdRH(i,j)=std(log(numberA(index)));        
        growtheggs=numberE(index); growtheggs=growtheggs(2:end)./growtheggs(1:end-1); % growth in absolute numbers where time step equals census step of 4 days
        growthjuvs=numberJ(index); growthjuvs=growthjuvs(2:end)./growthjuvs(1:end-1);
        growthadults=numberA(index); growthadults=growthadults(2:end)./growthadults(1:end-1);
        eggmeangrowthRH(i,j)=mean(log(growtheggs)); juvsmeangrowthRH(i,j)=mean(log(growthjuvs)); adultmeangrowthRH(i,j)=mean(log(growthadults));
        eggstdgrowthRH(i,j)=std(log(growtheggs));juvstdRH(i,j)=std(log(growthjuvs));adultstdRH(i,j)=std(log(growthadults));
        meaneggpropRH(i,j) = mean(numberE(index)./numberTotal); meanjuvpropRH(i,j) = mean(numberJ(index)./numberTotal); meanadultpropRH(i,j) = mean(numberA(index)./numberTotal);
        meanpopgrowthRH(i,j) = mean(log(numberTotal(2:end)./numberTotal(1:end-1)));
%hold on; subplot(3,1,1); plot(numberE(index)); hold on; subplot(3,1,2); plot(numberJ(index)); hold on; subplot(3,1,3); plot(numberA(index)); 
    end     
end

MeanResults(:,:,1) = eggmeannumberB; MeanResults(:,:,2) = eggmeannumberW; MeanResults(:,:,3) = eggmeannumberR; 
MeanResults(:,:,4) = juvsmeannumberB; MeanResults(:,:,5) = juvsmeannumberW; MeanResults(:,:,6) = juvsmeannumberR; 
MeanResults(:,:,7) = adultmeannumberB; MeanResults(:,:,8) = adultmeannumberW; MeanResults(:,:,9) = adultmeannumberR; 
MeanResults(:,:,10) = popmeanB; MeanResults(:,:,11) = popmeanW; MeanResults(:,:,12) = popmeanR; 

StdResults(:,:,1) = eggstdB; StdResults(:,:,2) = eggstdW; StdResults(:,:,3) = eggstdR; 
StdResults(:,:,4) = juvstdB; StdResults(:,:,5) = juvstdW; StdResults(:,:,6) = juvstdR; 
StdResults(:,:,7) = adultstdB; StdResults(:,:,8) = adultstdW; StdResults(:,:,9) = adultstdR; 
StdResults(:,:,10) = popstdB; StdResults(:,:,11) = popstdW; StdResults(:,:,12) = popstdR;

%% FIGURES
grey = [0.5,0.5,0.5];

% PREDICTED AND OBSERVED MEAN LOG COUNTS OF UNHARVESTED POPULATIONS

EvBmean = log(0.5*exp([5.16,5.16])); EvWmean = log(0.5*exp([5.29,5.29])); EvRmean = log(0.5*exp([5.12,5.12])); 
EvBupper = log(0.5*exp([5.22,5.22])); EvWupper = log(0.5*exp([5.37,5.37])); EvRupper = log(0.5*exp([5.30,5.30])); 
EvBlower = log(0.5*exp([5.10,5.10])); EvWlower = log(0.5*exp([5.20,5.20])); EvRlower = log(0.5*exp([4.92,4.92])); 
JvBmean = log(0.5*exp([5.77,5.77])); JvWmean = log(0.5*exp([5.75,5.75])); JvRmean = log(0.5*exp([5.52,5.52])); 
JvBupper = log(0.5*exp([5.80,5.80])); JvWupper = log(0.5*exp([5.82,5.82])); JvRupper = log(0.5*exp([5.70,5.70])); 
JvBlower = log(0.5*exp([5.74,5.74])); JvWlower = log(0.5*exp([5.68,5.68])); JvRlower = log(0.5*exp([5.34,5.34])); 
AvBmean = log(0.5*exp([4.32,4.32])); AvWmean = log(0.5*exp([4.36,4.36])); AvRmean = log(0.5*exp([4.33,4.33])); 
AvBupper = log(0.5*exp([4.35,4.35])); AvWupper = log(0.5*exp([4.41,4.41])); AvRupper = log(0.5*exp([4.45,4.45])); 
AvBlower = log(0.5*exp([4.29,4.29])); AvWlower = log(0.5*exp([4.31,4.31])); AvRlower = log(0.5*exp([4.20,4.20])); 
TvBmean = log(0.5*exp([6.36,6.36])); TvWmean = log(0.5*exp([6.41,6.41])); TvRmean = log(0.5*exp([6.22,6.22])); 
TvBupper = log(0.5*exp([6.39,6.39])); TvWupper = log(0.5*exp([6.46,6.46])); TvRupper = log(0.5*exp([6.38,6.38])); 
TvBlower = log(0.5*exp([6.34,6.34])); TvWlower = log(0.5*exp([6.36,6.36])); TvRlower = log(0.5*exp([6.06,6.06])); 

subplot(4,3,1); hold on; contour(E_Ymin,E_Ymax,eggmeannumberB,EvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberB,EvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,eggmeannumberB,EvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberB,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,2); contour(E_Ymin,E_Ymax,eggmeannumberW,EvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberW,EvWlower,'Color',grey,'LineWidth',2); 
contour(E_Ymin,E_Ymax,eggmeannumberW,EvWmean,'k','LineWidth',2); hold on; hold on; contour(E_Ymin,E_Ymax,eggmeannumberW,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,3); contour(E_Ymin,E_Ymax,eggmeannumberR,EvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberR,EvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,eggmeannumberR,EvRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberR,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,4); contour(E_Ymin,E_Ymax,juvsmeannumberB,JvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberB,JvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,juvsmeannumberB,JvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberB,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,5); contour(E_Ymin,E_Ymax,juvsmeannumberW,JvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberW,JvWlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,juvsmeannumberW,JvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberW,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,6); contour(E_Ymin,E_Ymax,juvsmeannumberR,JvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberR,JvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,juvsmeannumberR,JvRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberR,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,7); contour(E_Ymin,E_Ymax,adultmeannumberB,AvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberB,AvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberB,AvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberB,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,8); contour(E_Ymin,E_Ymax,adultmeannumberW,AvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberW,AvWlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberW,AvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberW,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,9); contour(E_Ymin,E_Ymax,adultmeannumberR,AvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberR,AvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberR,AvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberR,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,10); contour(E_Ymin,E_Ymax,popmeanB,TvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanB,TvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanB,TvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanB,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,11); contour(E_Ymin,E_Ymax,popmeanW,TvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanW,TvWlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanW,TvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanW,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,12); contour(E_Ymin,E_Ymax,popmeanR,TvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanR,TvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanR,TvRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanR,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 

% PREDICTED AND OBSERVED MEAN LOG COUNTS OF HARVESTED POPULATIONS

EvBmean = log(0.5*exp([4.94,4.94])); EvWmean = log(0.5*exp([4.89,4.89])); EvRmean = log(0.5*exp([5.15,5.15])); 
EvBupper = log(0.5*exp([5.03,5.03])); EvWupper = log(0.5*exp([4.99,4.99])); EvRupper = log(0.5*exp([5.24,5.24])); 
EvBlower = log(0.5*exp([4.86,4.86])); EvWlower = log(0.5*exp([4.8,4.8])); EvRlower = log(0.5*exp([5.07,5.07])); 
JvBmean = log(0.5*exp([5.6,5.6])); JvWmean = log(0.5*exp([5.59,5.59])); JvRmean = log(0.5*exp([5.57,5.57])); 
JvBupper = log(0.5*exp([5.65,5.65])); JvWupper = log(0.5*exp([5.64,5.64])); JvRupper = log(0.5*exp([5.64,5.64])); 
JvBlower = log(0.5*exp([5.54,5.54])); JvWlower = log(0.5*exp([5.52,5.52])); JvRlower = log(0.5*exp([5.49,5.49])); 
AvBmean = log(0.5*exp([4.04,4.04])); AvWmean = log(0.5*exp([3.93,3.93])); AvRmean = log(0.5*exp([3.99,3.99])); 
AvBupper = log(0.5*exp([4.1,4.1])); AvWupper = log(0.5*exp([3.99,3.99])); AvRupper = log(0.5*exp([4.05,4.05])); 
AvBlower = log(0.5*exp([3.98,3.98])); AvWlower = log(0.5*exp([3.87,3.87])); AvRlower = log(0.5*exp([3.92,3.92])); 
TvBmean = log(0.5*exp([6.16,6.16])); TvWmean = log(0.5*exp([6.15,6.15])); TvRmean = log(0.5*exp([6.22,6.22])); 
TvBupper = log(0.5*exp([6.21,6.21])); TvWupper = log(0.5*exp([6.2,6.2])); TvRupper = log(0.5*exp([6.29,6.29])); 
TvBlower = log(0.5*exp([6.12,6.12])); TvWlower = log(0.5*exp([6.09,6.09])); TvRlower = log(0.5*exp([6.16,6.16])); 

figure
subplot(4,3,1); contour(E_Ymin,E_Ymax,eggmeannumberBH,EvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberBH,EvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,eggmeannumberBH,EvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,2); contour(E_Ymin,E_Ymax,eggmeannumberWH,EvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberWH,EvWlower,'Color',grey,'LineWidth',2); 
hold on;contour(E_Ymin,E_Ymax,eggmeannumberWH,EvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,3); contour(E_Ymin,E_Ymax,eggmeannumberRH,EvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,eggmeannumberRH,EvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,eggmeannumberRH,EvRmean,'k','LineWidth',2); hold on;contour(E_Ymin,E_Ymax,eggmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,4); contour(E_Ymin,E_Ymax,juvsmeannumberBH,JvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberBH,JvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,juvsmeannumberBH,JvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,5); contour(E_Ymin,E_Ymax,juvsmeannumberWH,JvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberWH,JvWlower,'Color',grey,'LineWidth',2); 
hold on;contour(E_Ymin,E_Ymax,juvsmeannumberWH,JvWmean,'k','LineWidth',2); hold on;  contour(E_Ymin,E_Ymax,juvsmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,6); contour(E_Ymin,E_Ymax,juvsmeannumberRH,JvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberRH,JvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,juvsmeannumberRH,JvRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,juvsmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,7); contour(E_Ymin,E_Ymax,adultmeannumberBH,AvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberBH,AvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberBH,AvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,8); contour(E_Ymin,E_Ymax,adultmeannumberWH,AvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberWH,AvWlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberWH,AvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square;
subplot(4,3,9); contour(E_Ymin,E_Ymax,adultmeannumberRH,AvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,adultmeannumberRH,AvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,adultmeannumberRH,AvBmean,'k','LineWidth',2); hold on; hold on; contour(E_Ymin,E_Ymax,adultmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,10); contour(E_Ymin,E_Ymax,popmeanBH,TvBupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanBH,TvBlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanBH,TvBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,11); contour(E_Ymin,E_Ymax,popmeanWH,TvWupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanWH,TvWlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanWH,TvWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
subplot(4,3,12); contour(E_Ymin,E_Ymax,popmeanRH,TvRupper,'Color',grey,'LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanRH,TvRlower,'Color',grey,'LineWidth',2); 
hold on; contour(E_Ymin,E_Ymax,popmeanRH,TvRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,popmeanRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 

% PREDICTED AND OBSERVED POPULATION GROWTH RATES
vBmean = [0,0]; vBHmean = [-0.01,-0.01]; vWmean = [0.01,0.01]; vWHmean = [0,0]; vRmean = [0.01,0.01]; vRHmean = [0,0];
vBupper = [0.02,0.02]; vBHupper = [0.04,0.04]; vWupper = [0.06,0.06]; vWHupper = [0.05,0.05]; vRupper = [0.09,0.09]; vRHupper = [0.04,0.04];

figure
subplot(2,3,1); contour(E_Ymin,E_Ymax,meanpopgrowthB,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthB,vBmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthB,vBupper,'Color',grey,'LineWidth',2)
subplot(2,3,2); contour(E_Ymin,E_Ymax,meanpopgrowthW,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthW,vWmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthW,vWupper,'Color',grey,'LineWidth',2)
subplot(2,3,3); contour(E_Ymin,E_Ymax,meanpopgrowthR,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthR,vRmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthR,vRupper,'Color',grey,'LineWidth',2)
subplot(2,3,4); contour(E_Ymin,E_Ymax,meanpopgrowthBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthBH,vBHmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthBH,vBHupper,'Color',grey,'LineWidth',2)
subplot(2,3,5); contour(E_Ymin,E_Ymax,meanpopgrowthWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthWH,vWHmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthWH,vWHupper,'Color',grey,'LineWidth',2)
subplot(2,3,6); contour(E_Ymin,E_Ymax,meanpopgrowthRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
hold on; contour(E_Ymin,E_Ymax,meanpopgrowthRH,vRHmean,'k','LineWidth',2); hold on; contour(E_Ymin,E_Ymax,meanpopgrowthRH,vRHupper,'Color',grey,'LineWidth',2)


%v0 = [0,0];
%figure
%subplot(4,3,1); contour(E_Ymin,E_Ymax,eggmeannumberB-eggmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,eggmeannumberB-eggmeannumberBH,v0,'b','LineWidth',2)
%subplot(4,3,2); contour(E_Ymin,E_Ymax,eggmeannumberW-eggmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,eggmeannumberW-eggmeannumberWH,v0,'b','LineWidth',2)
%subplot(4,3,3); contour(E_Ymin,E_Ymax,eggmeannumberR-eggmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,eggmeannumberR-eggmeannumberRH,v0,'b','LineWidth',2)
%subplot(4,3,4); contour(E_Ymin,E_Ymax,juvsmeannumberB-juvsmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,juvsmeannumberB-juvsmeannumberBH,v0,'b','LineWidth',2)
%subplot(4,3,5); contour(E_Ymin,E_Ymax,juvsmeannumberW-juvsmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,juvsmeannumberW-juvsmeannumberWH,v0,'b','LineWidth',2)
%subplot(4,3,6); contour(E_Ymin,E_Ymax,juvsmeannumberR-juvsmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,juvsmeannumberR-juvsmeannumberRH,v0,'b','LineWidth',2)
%subplot(4,3,7); contour(E_Ymin,E_Ymax,adultmeannumberB-adultmeannumberBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,adultmeannumberB-adultmeannumberBH,v0,'b','LineWidth',2)
%subplot(4,3,8); contour(E_Ymin,E_Ymax,adultmeannumberW-adultmeannumberWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,adultmeannumberW-adultmeannumberWH,v0,'b','LineWidth',2)
%subplot(4,3,9); contour(E_Ymin,E_Ymax,adultmeannumberR-adultmeannumberRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,adultmeannumberR-adultmeannumberRH,v0,'b','LineWidth',2)
%subplot(4,3,10); contour(E_Ymin,E_Ymax,popmeanB-popmeanBH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,popmeanB-popmeanBH,v0,'b','LineWidth',2)
%subplot(4,3,11); contour(E_Ymin,E_Ymax,popmeanW-popmeanWH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,popmeanW-popmeanWH,v0,'b','LineWidth',2)
%subplot(4,3,12); contour(E_Ymin,E_Ymax,popmeanR-popmeanRH,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; %caxis manual; caxis([bottomLambda topLambda]);
%hold on; contour(E_Ymin,E_Ymax,popmeanR-popmeanRH,v0,'b','LineWidth',2)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [numberE, numberJ, numberA, numberTotal]= stochseq(Env,MatrixSize,E_Ymax,E_Ymin,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient,tlimit)

%%%% CODE TAKEN FROM TULJA et al 2003 Am Nat%%%%%%%

% specify intial population structure vector and reproductive value vector
vec1=15/nbins*ones(1,nbins); % 15 inds spread over all size bins
vec1=vec1'; 
popvecAbs = vec1;

trun=transient+tlimit; %time to run is desired length of sample path plus transients

%set up a new cumulative number for all trun
for i = 1:trun
    i2=Env(i);
         if i2==8, E_Y=E_Ymax; else E_Y=E_Ymin;
         end
    Linf = E_Y*Lm; % ultimate length  
    Lp = max(0.314,0.539*Linf); % length at puberty with minimum length of 0.314
    rb = 1/(beta + alpha*Linf); % von Bertellanfy growth rate
          
    [S, R, G, D, meshpts] = BigMatrixShrink(MatrixSize,Lmin,Lm,Lp,Rm,Lm,E_Y,E_Ystdev,rb,kappa,Lb,mu);
    mat1 = G*S + D*R;        
    vec1 = mat1 * vec1;
    numberE1 = sum(vec1.*(meshpts<1.1*Lmin)');
    numberJ1 = sum(vec1.*(meshpts>=1.1*Lmin & meshpts<Lp)');
    numberA1 = sum(vec1.*(meshpts>=Lp)');
    numberTotal1 = sum(vec1);
    %vec1= vec1/number1;
          
		if( i > transient)      %only store the ones after the transients
            i1 = i - transient;
            numberE(i1) = numberE1;
            numberJ(i1) = numberJ1;
            numberA(i1) = numberA1;
            numberTotal(i1) = numberTotal1;                
        end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [numberE, numberJ, numberA, numberTotal]= stochseqHarvest(Env, EnvHarvest, MatrixSize,E_Ymax,E_Ymin,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,nbins,transient,tlimit)
%%% CODE TAKEN FROM TULJA et al 2003 Am Nat%%%%%%%

% specify intial population structure vector and reproductive value vector
vec1=15/nbins*ones(1,nbins); % 15 inds spread over all size bins
vec1=vec1';  
trun=transient+tlimit; %time to run is desired length of sample path plus transients

for i = 1:trun
    i2=Env(i);
         if i2==8, E_Y=E_Ymax; else E_Y=E_Ymin;   end
    Linf = E_Y*Lm; % ultimate length  
    Lp = max(0.314,0.539*Linf); % length at puberty with minimum length of 0.314
    rb = 1/(beta + alpha*Linf); % von Bertellanfy growth rate
    [S, R, G, D, meshpts] = BigMatrixShrink(MatrixSize,Lmin,Lm,Lp,Rm,Lm,E_Y,E_Ystdev,rb,kappa,Lb,mu);                      
    
         % do harvesting first
         i3=EnvHarvest(i); 
            if (i3==1 && sum(meshpts>=Lp)>0), % % only harvest if there are adults            
            threshold=1.0235*wmean((meshpts(meshpts>=Lp))',vec1((nbins-length(meshpts(meshpts>Lp))+1):end)); % calculate harvest threshold size by multiplying weighted adult size times 1.0235
            vec1(meshpts>=threshold)=0.5*vec1(meshpts>=threshold); % % harvesting is corrected by the threshold to harvest 50% of total number of large females
            % then calculate population for next time step after harvesting
            mat1 = G*S + D*R;
            vec1 = mat1* vec1;
            numberE1 = sum(vec1.*(meshpts<1.1*Lmin)');
            numberJ1 = sum(vec1.*(meshpts>=1.1*Lmin & meshpts<Lp)');
            numberA1 = sum(vec1.*(meshpts>=Lp)');
            numberTotal1 = sum(vec1);
         else
         % calculate population for next time step without harvesting        
            mat1 = G*S + D*R;
            vec1 = mat1* vec1;
            numberE1 = sum(vec1.*(meshpts<1.1*Lmin)');
            numberJ1 = sum(vec1.*(meshpts>=1.1*Lmin & meshpts<Lp)');
            numberA1 = sum(vec1.*(meshpts>=Lp)');
            numberTotal1 = sum(vec1);
         end
                   
		if( i > transient)      %only store the ones after the transients
            i1 = i - transient;
            numberE(i1) = numberE1;
            numberJ(i1) = numberJ1;
            numberA(i1) = numberA1;
            numberTotal(i1) = numberTotal1;                    
        end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = wmean(x,w,dim)
%WMEAN   Weighted Average or mean value.
%   For vectors, WMEAN(X,W) is the weighted mean value of the elements in X
%   using non-negative weights W. For matrices, WMEAN(X,W) is a row vector 
%   containing the weighted mean value of each column.  For N-D arrays, 
%   WMEAN(X,W) is the weighted mean value of the elements along the first 
%   non-singleton dimension of X.
%
%   Each element of X requires a corresponding weight, and hence the size 
%   of W must match that of X.
%
%   WMEAN(X,W,DIM) takes the weighted mean along the dimension DIM of X. 
%
%   Class support for inputs X and W:
%      float: double, single
%
%   Example:
%       x = rand(5,2);
%       w = rand(5,2);
%       wmean(x,w)

if nargin<2
    error('Not enough input arguments.');
end

% Check that dimensions of X match those of W.
if(~isequal(size(x), size(w)))
    error('Inputs x and w must be the same size.');
end

% Check that all of W are non-negative.
if (any(w(:)<0))
    error('All weights, W, must be non-negative.');
end

% Check that there is at least one non-zero weight.
if (all(w(:)==0))
    error('At least one weight must be non-zero.');
end

if nargin==2, 
  % Determine which dimension SUM will use
  dim = min(find(size(x)~=1));
  if isempty(dim), dim = 1; end
end

y = sum(w.*x,dim)./sum(w,dim);