function [p, q, Matrix_lambda_sens, Matrix_lambda_sec_sens]  = stochseq_DEBIPM_sens_bulbmites(E_Ygood, E_Ybad);
% Lb, Lp, Lm, kappa, mu, rb, Rm)
steps = 15;
% p: chance of moving from good to bad environment
% q; % chance of moving from bad to good environment
p=linspace(0,1,steps); q = linspace(0,1,steps);

transient=200; % initial transient of length
tlimit=1500; % Number of time steps
trun=transient+tlimit; %time to run is desired length of sample path plus transients

MatrixSize = 200; % number of columns and rows

% DEB IPM pars 
Lmin = 150/1000; % minimum body length in mm (used to define limits of matrix approximation)
Lb = 166/1000; % length at birth in mm 
Lm = 1008/1000; % maximum body length in mm 
E_Ystdev=0.3; % sigma(Y)
kappa = 0.082; % fraction energy allocation to respiration (as opposed to reproduction)
mu = 0.03; % mortality rate
beta = 151; % to calculate von Bertalanffy growth rate r_b
alpha = -137.8; % to calculate von Bertalanffy growth rate r_b
Rm = 32; % maximum reproduction rate of female offspring at L_m

fracperturb = 0.01; % perturbation of DEB pars
perturb=ones(1:7); % Lb, Lp, Lm, kappa, mu, rb, Rm)

for k = 1:length(p), disp(['lambda' num2str(k/length(p))]) % rows/ yaxis, p
    for l = 1:length(q) % columns / xaxis, q
        % habitat transition matrix H; 1=good; 2=bad; p379 Caswell 2001
        H = [1-p(k) q(l); 
             p(k) 1-q(l)];
        % make Markov chain of environments
        Env=habitat_matrix(H,trun);
        %calculate vector of unperturbed stochastic growh rate
        [growth, lambda_s_orig] = stochseq(Env,E_Ygood,E_Ybad, MatrixSize,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,transient,tlimit);        
        for i=1:length(perturb);
           perturbation = ones(1:7); perturbation(i) = perturbation(i) + fracperturb;
           [growth_pert, lambda_s_pert] = stochseqperturb(Env,E_Ygood,E_Ybad, MatrixSize,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,transient,tlimit, perturbation);
        
           Lambda_s_perturb(i,1) = lambda_s_pert;
        end
        %calculate maximum sensitivity of stochastic growth rate to perturbations
        Lambda_s_Sens = (Lambda_s_perturb - lambda_s_orig)./fracperturb;
        Lambda_s_Sens = abs(Lambda_s_Sens); % use absolute elasticity values
        Lambda_s_sec_Sens = Lambda_s_Sens; % also explore second highest elasticity value
        [a b] = max(Lambda_s_Sens);
        if b==1, Matrix_lambda_sens(k,l) = 1; Lambda_s_sec_Sens(b)=-10; 
        elseif b==2; Matrix_lambda_sens(k,l)=2;  Lambda_s_sec_Sens(b)=-10;
        elseif b==3, Matrix_lambda_sens(k,l)=3;  Lambda_s_sec_Sens(b)=-10;
        elseif b==4, Matrix_lambda_sens(k,l)=4;  Lambda_s_sec_Sens(b)=-10;
        elseif b==5, Matrix_lambda_sens(k,l)=5;  Lambda_s_sec_Sens(b)=-10;
        elseif b==6, Matrix_lambda_sens(k,l)=6; Lambda_s_sec_Sens(b)=-10;
        else Matrix_lambda_sens(k,l)=7;  Lambda_s_sec_Sens(b)=-10; end; % rows are p (yaxis) and colums are q (xaxis)            
    %calculate second maximum sensitivity of stochastic growth rate to perturbations
        [a b] = max(Lambda_s_sec_Sens);
        if b==1, Matrix_lambda_sec_sens(k,l) = 1; elseif b==2; Matrix_lambda_sec_sens(k,l)=2; elseif b==3, Matrix_lambda_sec_sens(k,l)=3; 
        elseif b==4, Matrix_lambda_sec_sens(k,l)=4; elseif b==5, Matrix_lambda_sec_sens(k,l)=5; elseif b==6, Matrix_lambda_sec_sens(k,l)=6;
        else Matrix_lambda_sec_sens(k,l)=7; end; % rows are p (yaxis) and colums are q (xaxis)          
    end
end


figure
contour(q,p,Matrix_lambda_sens,'k','Showtext','on'); set(gca,'ydir','normal'); axis square; 
% hold on; contour(q,p,Matrix_ls,[0,0],'r','LineWidth',2); 
xlabel('q: probability of moving to good environment')
ylabel('p: probability of moving to bad environment')
title('elasticity of lambda_s')
figure
imagesc(q,p,Matrix_lambda_sens); colorbar; set(gca,'ydir','normal');  % q: x-axis; p: y-axis
xlabel('q: probability of moving to good environment')
ylabel('p: probability of moving to bad environment')
title('elasticity of lambda_s')
%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%

function [growth, lam_S]= stochseq(Env,E_Ygood, E_Ybad, MatrixSize,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,transient,tlimit)

% specify intial population structure vector and reproductive value vector
vec1=15/MatrixSize*ones(1,MatrixSize); % 15 inds spread over all size bins
vec1=vec1'; 
popvecAbs = vec1;

trun=transient+tlimit; %time to run is desired length of sample path plus transients

%set up a new cumulative number for all trun
for i = 1:trun
    i2=Env(i);
         if i2==1, E_Y=E_Ygood; else E_Y=E_Ybad;
         end
    Linf = E_Y*Lm; % ultimate length  
    Lp = max(0.314,0.539*Linf); % length at puberty with minimum length of 0.314
    rb = 1/(beta + alpha*Linf); % von Bertellanfy growth rate
          
    [S, R, G, D, meshpts] = BigMatrixShrink(MatrixSize,Lmin,Lm,Lp,Rm,Lm,E_Y,E_Ystdev,rb,kappa,Lb,mu);
    mat1 = G*S + D*R;        
    vec1 = mat1 * vec1;
    %numberE1 = sum(vec1.*(meshpts<1.1*Lmin)');
    %numberJ1 = sum(vec1.*(meshpts>=1.1*Lmin & meshpts<Lp)');
    %numberA1 = sum(vec1.*(meshpts>=Lp)');
    growth1 = sum(vec1);
    vec1= vec1/growth1;
          
		if( i > transient)      %only store the ones after the transients
            i1 = i - transient;
            growth(i1) = growth1;                
        end
end
lam_S=mean(log(growth)); lam_S=exp(lam_S);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [growth, lam_S]= stochseqperturb(Env,E_Ygood, E_Ybad, MatrixSize,Lmin,Rm,Lm,E_Ystdev,kappa,Lb,mu,beta,alpha,transient,tlimit,perturbation)

% specify intial population structure vector and reproductive value vector
vec1=15/MatrixSize*ones(1,MatrixSize); % 15 inds spread over all size bins
vec1=vec1'; 
popvecAbs = vec1;

trun=transient+tlimit; %time to run is desired length of sample path plus transients

%set up a new cumulative number for all trun
for i = 1:trun
    i2=Env(i);
         if i2==1, E_Y=E_Ygood; else E_Y=E_Ybad;
         end
    Linf = E_Y*Lm; % ultimate length  
    Lp = max(0.314,0.539*Linf); % length at puberty with minimum length of 0.314
    rb = 1/(beta + alpha*Linf); % von Bertellanfy growth rate
          
    [S, R, G, D, meshpts] = BigMatrixShrinkSens(MatrixSize,Lmin,Lm,Lp,Rm,Lm,E_Y,E_Ystdev,rb,kappa,Lb,mu,perturbation);
    mat1 = G*S + D*R;        
    vec1 = mat1 * vec1;
    %numberE1 = sum(vec1.*(meshpts<1.1*Lmin)');
    %numberJ1 = sum(vec1.*(meshpts>=1.1*Lmin & meshpts<Lp)');
    %numberA1 = sum(vec1.*(meshpts>=Lp)');
    growth1 = sum(vec1);
    vec1= vec1/growth1;
          
		if( i > transient)      %only store the ones after the transients
            i1 = i - transient;
            growth(i1) = growth1;                
        end
end
lam_S=mean(log(growth)); lam_S=exp(lam_S);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%